#!/usr/bin/env bash

# common.sh - common functions.
# Copyright (c) 2022 ge <https://nixhacks.net/>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

log() {
    # Logger. Write message to __log_file.
    #
    # Usage: log [-pV] MESSAGE
    #
    # Can read from STDIN. Removes ANSI escape codes from log message.
    # Options:
    # -p    echo message to STDOUT too
    # -V    do not print message to STDOUT in verbose mode

    local message
    local print
    local ignore_verbose

    # Read message from STDIN if args not passed.
    [[ ! -t 0 ]] || [[ "$#" == 0 ]] && message="$(cat <&0)"

    # Set log date format
    log_date_format="${log_date_format:-%d/%b/%Y:%H:%M:%S %z}"

    while (( "$#" )); do
        case "$1" in
            -p) print=1; shift;;
            -V) ignore_verbose=1; shift;;
            -)  message="$(cat <&0)"; shift "$#";;
            *)  message="$*"; shift "$#";;
        esac
    done

    if [ -n "$print" ] || [ -n "$__verbose" ]; then
        if [ -n "$print" ] || [ -z "$ignore_verbose" ]; then
            echo -e "$message"
        fi
    fi

    echo -e "$message" |
    sed -r 's/\x1B\[(([0-9]+)(;[0-9]+)*)?[m,K,H,f,J]//g' |
    while read -r line; do
        if [ -n "$line" ]; then
            # shellcheck disable=SC2154
            printf '[%s] %s\n' \
                "$(date +"$log_date_format")" "$line" >> "$__log_file"
        fi
    done
}

handle_error() {
    # Handle errors
    #
    # Usage: handle_error ERROR_MESSAGE

    # Display amd log message
    log -p "$*" >&2
    log -p "Check $__log_file log for details." >&2

    # Run user defined error handler function if set
    if is_function_set on_error; then
        echo -e "Run \e[1mon_error()\e[0m" | log -p
        on_error "$*"
    fi

    log -p "Backup ERROR: Exiting with previous errors" >&2; exit 1
}

is_installed() {
    # Test is the program installed.
    # See good answer: https://stackoverflow.com/a/677212
    #
    # Usage: is_installed COMMAND

    if ! command -v "$1" >/dev/null 2>&1; then
        echo "Error: Command $1 not found." \
            "Please install $1 or check your" \
            "PATH if it's actually installed." >&2
        exit 1
    fi
}

is_function_set() {
    # Test is function set or not. Return exit code.
    # Useful with 'if' statement.
    #
    # Usage: is_function_set FUNCTION

    declare -F -- "$1" > /dev/null
    return "$?"
}

remove_if_empty() {
    # Delete empty file
    #
    # Usage: remove_empty FILE

    if [ -f "$1" ]; then
        if [ ! -s "$1" ]; then
            log -p "Backup size is 0 bytes. Removing $1" >&2
            rm "$1"
        fi
    fi
}

set_compression() {
    # Set `compr_cmd` and `compr_ext` by name.

    # Select compression utility and set filename extension.
    # Refference: https://www.gnu.org/software/tar/manual/html_node/gzip.html
    case "$1" in
        gzip|gz)    compr_ext='.gz';      compr_cmd='gzip';;
        tgz)        compr_ext='.gz';      compr_cmd='gzip';;
        taz)        compr_ext='.gz';      compr_cmd='gzip';;
        compress|Z) compr_ext='.Z';       compr_cmd='compress';;
        taZ)        compr_ext='.Z';       compr_cmd='compress';;
        bzip2|bz2)  compr_ext='.bz2';     compr_cmd='bzip2';;
        tz2)        compr_ext='.bz2';     compr_cmd='bzip2';;
        tbz2)       compr_ext='.bz2';     compr_cmd='bzip2';;
        tbz)        compr_ext='.bz2';     compr_cmd='bzip2';;
        lzip|lz)    compr_ext='.lz';      compr_cmd='lzip';;
        lzma)       compr_ext='.lzma';    compr_cmd='lzma';;
        tlz)        compr_ext='.lzma';    compr_cmd='lzma';;
        lzop|lzo)   compr_ext='.lzop';    compr_cmd='lzop';;
        xz)         compr_ext='.xz';      compr_cmd='xz';;
        zstd|zst)   compr_ext='.zst';     compr_cmd='zstd --rm';;
        tzst)       compr_ext='.zst';     compr_cmd='zstd --rm';;
        *)  # No compression
            # shellcheck disable=SC2034
            compr_ext=
            # shellcheck disable=SC2034
            compr_cmd=
            ;;
    esac

    [ -z "$compr_cmd" ] && return # Exit from function if no compression set

    is_installed "${compr_cmd%% *}"
}

compress_file() {
    # Compress and echo compressed file name. Remove uncompressed file.
    #
    # Usage: compress_file FILE

    local compressed

    # Select compression utility and set filename extension.
    if [ -n "$compression" ]; then
        set_compression "$compression"
    else
        compr_cmd=
        compr_ext=
    fi

    # Compress file
    if [ -n "$compr_cmd" ]; then
        log -V "Compressing file $1 by ${compr_cmd%% *} ..."
        $compr_cmd "$1" 2>> "$__log_file"
        compressed="${1}${compr_ext}"
    fi

    if [ -f "$compressed" ]; then
        if [ -f "$1" ]; then
            log "Remove uncompressed file $1"
            rm -- "$1"
        fi
    fi

    echo "$compressed"
}

gen_backup_name() {
    # Generate backup file name. Write resulting string to STDOUT.
    #
    # Usage: gen_backup_name NAME_EXT

    local name
    local name_ext

    name_prefix="${name_prefix:-${backup_script##*/}_}"
    # shellcheck disable=SC2154
    name="$(basename "$path")" # `path` is variable parsed from URI
    name_ext="$1"
    name_date_format="${name_date_format:-_%Y.%m.%d}"
    name_suffix="${name_suffix:--%H%M}"

    date +"${name_prefix}${name}${name_date_format}${name_suffix}${name_ext}"
}
